@@ -5,7 +5,7 @@
 
 START_EXTERN_C
 
-typedef struct OPAnnotationGroup *OPAnnotationGroup;
+typedef struct OPAnnotationGroupImpl *OPAnnotationGroup;
 typedef  OP *(*OPAnnotationPPAddr)(pTHX);
 typedef void (*OPAnnotationDtor)(pTHX_ void *data);
 
@@ -16,8 +16,10 @@ typedef struct {
 } OPAnnotation;
 
 OPAnnotationGroup op_annotation_group_new();
-OPAnnotation * op_annotation_new(OPAnnotationGroup group, OPAnnotationPPAddr ppaddr, void *data, OPAnnotationDtor dtor);
 void op_annotation_group_free(OPAnnotationGroup group);
+OPAnnotation * op_annotation_get(OPAnnotationGroup group, OP *op);
+OPAnnotation * op_annotation_set(OPAnnotationGroup group, OP *op, void *data, OPAnnotationDtor dtor);
+void op_annotation_delete(OPAnnotationGroup table, OP * annotation);
 void op_annotation_free(OPAnnotation *annotation);
 
 END_EXTERN_C
